/*****************************************************************************
 *
 * Author: Steve Dunn, MIT Wearable Computing Group
 * http://www.media.mit.edu/wearables/
 * Copyright (C) 2002 MIT Media Lab
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <stdlib.h>
#include <stdio.h>

#include "Ant.hpp"
#include "Classification.hpp"

class MyHandler : public ContextHandler {
protected:
	Classification classification;
public:
	Label *label;
	char *nodeName;

	MyHandler() {};
	virtual ~MyHandler() {}
	virtual void handleUpdate(NodeType *node, char *unl);
};

void MyHandler::handleUpdate(NodeType *updateNode, char *unl) {
	if(updateNode == NULL)
		return;

	if(strcmp(updateNode->name, nodeName) == 0) {
		// printf("Got Accel Node\n");

		classification.load(updateNode);
		// classification.show();

		label->setVal( classification.mostLikely() );
	}
	else {
		printf("Got Context Update: %s\n", updateNode->name);
	}
}

int main(int argc, char *argv[]) {
	MainApp mainApp;
	MainWin mainWin;
	Label label;
	MyHandler handler;
	char *unl;
	char *nodeName;

    if(argc >= 3) {
        mainApp.setServerName(argv[1]);
    }
    else {
        printf("Usage: AccelApp [server name] [unl]\n");
        exit(0);
    }

	unl = argv[2];
    if(mainApp.context != NULL) {
        mainApp.context->subscribeToNode(unl);
        mainApp.setContextHandler(&handler);
    }

    nodeName = strrchr(unl, '/');
    if(nodeName == NULL) {
        printf("Error extracting node name from partial unl\n");
        exit(1);
    }
    nodeName++;

	handler.nodeName = nodeName;

	mainWin.makeApplet(1);
	mainWin.setName("Motion Classifier");

	label.setVal("AccelApp");
	mainWin.addChild(&label);

	handler.label = &label;

	mainWin.showAll();

	mainApp.mainLoop();

	return 0;
}

