/*****************************************************************************
 *
 * Author: Steve Dunn, Rich DeVaul, MIT Wearable Computing Group
 * http://www.media.mit.edu/wearables/
 * Copyright (C) 2002 MIT Media Lab
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/time.h>
#include <signal.h>
#include <math.h>

#include <fcntl.h>
#include <errno.h>
#include <sys/ioctl.h>

#include "Context.hpp"

#include "RealVector.hpp"
#include "SignalGen.hpp"

volatile int runflag = 1;

void handler(int foo) {
  switch (foo) {
  case SIGALRM:
    //    puts("Signal handler: Alarm received");
    break;
  case SIGINT:
    puts("Signal handler: SIGINT (keyboard interrupt)");
    runflag=0;
exit(1);
    break;
  default:
    printf("Signal handler received %d\n",foo);
  }
  return;
}

int main(int argc, char *argv[]) {
  SignalGen *signalGen;
  RealVector *feature;
  char *sigUNL = "/signals/accel";
  int i2cdev;
  int accelID = 88;
  signed char buffer[4];
  int count;
  Context *context = NULL;
  struct itimerval *value =
    (struct itimerval *)(malloc(sizeof(struct itimerval)));;

  long loopTime = 20905l - 10000l; // something like 10ms overhead

  //  long loopTime = 31250l - 10000l; // something like 10ms overhead?
  float x, y, z;

  if(argc >= 2) {
    context = new Context(argv[1]);
  }
  else {
    printf("Usage: AccelSignal [server name]\n");
    exit(0);
  }

  printf("AccelSignal\n");

	/**** Open connection to I2C device ****/
  i2cdev = open("/dev/i2c-0", O_RDWR);
  if(i2cdev < 0) {
    printf("Cannot connect to I2C device. Exiting.\n");
    exit(1);
  }
  if(ioctl(i2cdev, 0x0703, accelID) < 0) {
    printf("Error selecting accelerometer device. Exiting\n");
    exit(1);
  }
  
  signalGen = new SignalGen(sigUNL, context);
  if(signalGen == NULL) {
    printf("Error creating SignalGen\n");
    exit(1);
  }

  feature = new RealVector(4);
  if(feature == NULL) {
    printf("Error creating feature\n");
    exit(1);
  }
				// Install signal handler
  signal(SIGALRM,handler);
  signal(SIGINT,handler);
				// Set timer value and reset value
  value->it_interval.tv_sec=0;
  value->it_interval.tv_usec=loopTime;
  value->it_value.tv_sec=0;
  value->it_value.tv_usec=loopTime;
				// Set real-time timer
  setitimer(ITIMER_REAL,value,NULL);
				// Until we are interrupted...
  while(runflag) {
    signalGen->getConnections(0);

    if(0 || signalGen->hasListeners()) {
	// wait for alarm signal
      pause();

      buffer[0] = 2;
      count = write(i2cdev, buffer, 1);
      if(count != 1) {
	perror("write i2c");
	fprintf(stderr, "write error %d - %d\n", count, errno);
	continue;
      }

      count = read(i2cdev, buffer, 4);
      if(count != 4) {
	perror("read i2c");
	fprintf(stderr, "read error %d - %d\n", count, errno);
	continue;
      }
      
      x = (float) buffer[0];
      y = (float) buffer[1];
      z = ((float)buffer[2]+(float)buffer[3])/2.0f;

      feature->setElement(0, sqrt(x*x+y*y+z*z) );
      feature->setElement(1, x);
      feature->setElement(2, y);
      feature->setElement(3, z);

// feature->show();
      signalGen->writeReals(feature);
    }
	else {
		sleep(1);
	}
  }

  close(i2cdev);
  delete feature;
  delete signalGen;

  return 0;
}

