/*****************************************************************************
 *
 * Author: Steve Dunn, MIT Wearable Computing Group
 * http://www.media.mit.edu/wearables/
 * Copyright (C) 2002 MIT Media Lab
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "ActionVisualMsg.hpp"
#include "Trigger.hpp"

ActionVisualMsg::ActionVisualMsg() {
	type = ACTION_VISUAL_MSG;
	filename = NULL;
	msg = NULL;
}

ActionVisualMsg::ActionVisualMsg(char *newFilename, char *newMsg) {
	type = ACTION_VISUAL_MSG;
	filename = strdup(newFilename);
	msg = strdup(newMsg);
}

ActionVisualMsg::~ActionVisualMsg() {
	if(filename != NULL)
		free(filename);
	if(msg != NULL)
		free(msg);
}

void ActionVisualMsg::show() {
	if(filename == NULL || msg == NULL)
		printf("ActionVisualMsg\n");
	else
		printf("ActionVisualMsg: '%s' - '%s'\n", filename, msg);
}

//-----------------------------

void ActionVisualMsg::setFilename(char *newFilename) {
	if(filename != NULL)
		free(filename);

	filename = strdup(newFilename);
}

char* ActionVisualMsg::getFilename() {
	return filename;
}


//-----------------------------


void ActionVisualMsg::setMsg(char *newMsg) {
	if(msg != NULL)
		free(msg);

	msg = strdup(newMsg);
}

char* ActionVisualMsg::getMsg() {
	return msg;
}


//-----------------------------

void ActionVisualMsg::write(FILE *fp) {
	if(filename == NULL || msg == NULL)
		fprintf(fp, "%d\t\t", type);
	else
		fprintf(fp, "%d\t%s\t%s", type, filename, msg);
}

//-----------------------------

void ActionVisualMsg::load(char *data) {
	char *field[2];

	if(Trigger::split(data, field, 2, NULL) == 2) {
		setFilename(field[0]);
		setMsg(field[1]);
	}
}

