/*****************************************************************************
 *
 * Author: Steve Dunn, MIT Wearable Computing Group
 * http://www.media.mit.edu/wearables/
 * Copyright (C) 2002 MIT Media Lab
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/time.h>
#include <math.h>

#include <fcntl.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <sys/soundcard.h>


#include "Context.hpp"
#include "RealVector.hpp"
#include "SignalGen.hpp"

#define BUF_SIZE 1

int main(int argc, char *argv[]) {
	SignalGen *signalGen;
	RealVector *feature;
	char *sigUNL = "/signals/audio";
	int audio_dev;
	char buffer[BUF_SIZE];
	int count;
	Context *context = NULL;
	struct timeb startTime, endTime;
	long loopTime = 20905l;
	long interval;
	// int i;
	// long speed = 11025;

    if(argc >= 2) {
        context = new Context(argv[1]);
    }
    else {
        printf("Usage: AudioSignal [server name]\n");
        // exit(0);
    }

	printf("AudioSignal\n");

	/**** Open connection to audio device ****/
	audio_dev = open("/dev/audio", O_RDONLY);
	if(audio_dev < 0) {
		printf("Cannot connect to I2C device. Exiting.\n");
		exit(1);
	}
/*
	if(ioctl(audio_dev, SNDCTL_DSP_SPEED, &speed) < 0) {
		printf("Error setting Sampling Rate. Exiting\n");
		exit(1);
	}
*/
  
	signalGen = new SignalGen(sigUNL, context);
	if(signalGen == NULL) {
		printf("Error creating SignalGen\n");
		exit(1);
	}

	feature = new RealVector(BUF_SIZE);
	if(feature == NULL) {
		printf("Error creating feature\n");
		exit(1);
	}

	ftime(&startTime);
	while(1) {
		signalGen->getConnections(0);

		if(1 || signalGen->hasListeners()) {
			ftime(&endTime);
			interval = (endTime.time*1000+endTime.millitm) - (startTime.time*1000+startTime.millitm);
			// printf("Time: %ld - %ld - %ld\n", loopTime , interval, loopTime - (1000*interval));
			if( (loopTime - (1000*interval)) > 0)
				usleep(loopTime - 1000*( (endTime.time*1000+endTime.millitm) - (startTime.time*1000+startTime.millitm)));
			else
				printf("Over timeslice\n");
			ftime(&startTime);

			count = read(audio_dev, buffer, BUF_SIZE);
			if(count != BUF_SIZE) {
				perror("read /dev/audio");
				fprintf(stderr, "read error %d - %d\n", count, errno);
				continue;
			}

			// for(i=0; i<BUF_SIZE; i++) 
				// feature->setElement(i, buffer[i]);
			feature->setElement(0, buffer[0]);
			

feature->show();
			signalGen->writeReals(feature);
		}
	}

	close(audio_dev);
	delete feature;
	delete signalGen;

	return 0;
}

