/*****************************************************************************
 *
 * Author: Steve Dunn, MIT Wearable Computing Group
 * http://www.media.mit.edu/wearables/
 * Copyright (C) 2002 MIT Media Lab
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "Context.hpp"
#include "ContextHandler.hpp"
#include "Classification.hpp"

class MyHandler : public ContextHandler {
public:
	char *node;
    virtual void handleUpdate(NodeType *node, char *unl);
};

void MyHandler::handleUpdate(NodeType *updateNode, char *unl) {
	Classification classification;

    if(updateNode == NULL)
        return;

	if(strcmp(unl, node) == 0) {
		classification.load(updateNode);
		// classification.show();

		printf("%ld: %s\n", time(NULL), classification.mostLikely());
	}
	else {
		printf("Unknown Node: %s - %s\n", node, unl);
	}
}

int main(int argc, char *argv[]) {
	Context *context = NULL;
	MyHandler handler;
	char unl[512];

    if(argc >= 3) {
        context = new Context(argv[1]);
		context->contextHandler = &handler;
    }
    else {
        printf("Usage: ClassLogger [server name] [unl]\n");
        exit(0);
    }

	sprintf(unl, "%s:%s", argv[1], argv[2]);
	handler.node = unl;

	context->subscribeToNode(unl);

	printf("ClassLogger\n");

    while( context->waitForNode() ) {
    	context->readNodes();
    }

	return 0;
}

