/*****************************************************************************
 *
 * Author: Steve Dunn, MIT Wearable Computing Group
 * http://www.media.mit.edu/wearables/
 * Copyright (C) 2002 MIT Media Lab
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>

#include "RealVector.hpp"
#include "SignalGen.hpp"
#include "Context.hpp"

int main(int argc, char *argv[]) {
	SignalGen *signalGen;
	RealVector *feature;
	char *sigUNL; // (ex. "/signals/test")
	Context *context = NULL;
	char *filename;
	FILE *fp;
	char buffer[256];
	int bufSize = 255;
	int dataLeft = 1;
	int i;
	int dimension;
	unsigned long  numRecords;

    if(argc >= 5) {
        context = new Context(argv[1]);
		filename = argv[2];
    }
    else {
        printf("Usage: FileSignal [server name] [filename] [dimension] [output unl]\n");
        exit(0);
    }

	printf("FileSignal\n");

	fp = fopen(filename, "r");
	if(fp == NULL) {
		perror("fopen");
		printf("Error opening file '%s'\n", filename);
		exit(1);
	}

	dimension = atoi(argv[3]);
	sigUNL = argv[4];

	signalGen = new SignalGen(sigUNL, context);
	if(signalGen == NULL) {
		printf("Error creating SignalGen\n");
		exit(1);
	}

	feature = new RealVector(dimension);
	if(feature == NULL) {
		printf("Error creating feature\n");
		exit(1);
	}

	numRecords = 0;
    while(dataLeft) {
		signalGen->getConnections(0);
		if(! signalGen->hasListeners()) 
			usleep(200);

		if(dataLeft && signalGen->hasListeners()) {
			for(i=0; i<dimension; i++) {
				if(fgets(buffer, bufSize, fp) == NULL) {
					dataLeft = 0;
					break;
				}

				feature->setElement(i, atof(buffer));
			}

			signalGen->writeReals(feature);
// feature->show();
			if((numRecords % 2000) == 0)
				printf("%ld records\n", numRecords);
			numRecords++;
		}
	}

	delete feature;
	delete signalGen;

	fclose(fp);

	printf("%ld Records written\n", numRecords);

	return 0;
}

