/*****************************************************************************
 *
 * Author: Steve Dunn, MIT Wearable Computing Group
 * http://www.media.mit.edu/wearables/
 * Copyright (C) 2002 MIT Media Lab
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>

#include "Context.hpp"
#include "ContextHandler.hpp"
#include "Signal.hpp"
#include "RealVector.hpp"
#include "Classification.hpp"
#include "Gaussian.hpp"
#include "MixtureModel.hpp"

int main(int argc, char *argv[]) {
	char *sigUNL; // (ex. /signals/accel)
	char *classUNL;   // (ex. /classifiers/accel)

	Context *context = NULL;
	Signal *signal;
	RealVector *feature;
	Classification *classification;

	MixtureModel mixtureModel;
	char *dataFile;

	printf("GClassifier\n");
    if(argc < 5) {
        printf("Usage: GClassifier [server name] [gaussian datafile] [input signal] [output unl]\n");
        exit(0);
    }

    context = new Context(argv[1]);
	dataFile = argv[2];
	sigUNL = argv[3];
	classUNL = argv[4];
	
	mixtureModel.load(dataFile);

	classification = new Classification();
	signal = new Signal(sigUNL, context);

	printf("Reading Signals --------\n");
	while(1) {
		if( (feature = signal->readReals()) != NULL ) {
			feature->show();

			// process feature vector here
			mixtureModel.classify(classification, feature);

			if(context != NULL)
				classification->publish(context, classUNL);
			delete feature;
			printf("\n");
		}
		else {
			printf("Sleeping\n");
			sleep(1);
		}
	}

	if(context != NULL) 
		delete context;
	delete signal;
	delete classification;

	return 0;
}

