/*****************************************************************************
 *
 * Author: Steve Dunn, MIT Wearable Computing Group
 * http://www.media.mit.edu/wearables/
 * Copyright (C) 2002 MIT Media Lab
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "Gaussian.hpp"
#include "RealVector.hpp"
#include "Matrix.hpp"

Gaussian::Gaussian() {
	identifier = 0;
	label = NULL;
	dimension = 0;
	prior = 0.0;
	sigma_det = 0.0;
	u = NULL;
	sigma_inv = NULL;

	_Verbose = 0;
}

Gaussian::~Gaussian() {
}

double Gaussian::calc(RealVector *feature) {
    RealVector feature_norm(dimension);
    RealVector result(dimension);
    double coeff;
	double data;

    if(feature->getNumElements() != dimension ||
       u->getNumElements() != dimension ||
       sigma_inv->getNumRows() != dimension ||
       sigma_inv->getNumCols() != dimension) {
        printf("Error - invalid vector and matrix dimensions do not match feature vector dimensions\n");
		printf("dimension: %d RealVector: %d u: %d sigma_inv rows: %d sigma_inv cols: %d\n",
			dimension, feature->getNumElements(), u->getNumElements(), sigma_inv->getNumRows(), sigma_inv->getNumCols());
        return 0;
    }

    // coeff = 1.0 / sqrt( pow(2*M_PI,dimension) * sigma_det);
    // prob = coeff * exp( -0.5 * (X-u) . sigma_inv . (X-u) );

	if(_Verbose) {
		printf("Label: %s\n", label);
		printf("Det: %e\n", sigma_det);
	}
    coeff = 1.0 / sqrt( pow(2*M_PI,dimension) * sigma_det);
	if(_Verbose)
		printf("Coeff: %e\n", coeff);
    RealVector::subtract(feature, u, &feature_norm);
	if(_Verbose) {
		printf("Subtract\n");
		feature_norm.show();
	}
    RealVector::multiply(&feature_norm, sigma_inv, &result);
	if(_Verbose) {
		printf("Multiply\n");
		result.show();
	}
	data = RealVector::innerProduct(&result, &feature_norm);
	if(_Verbose)
		printf("innerProduct: %e\n", data);

	data *= -0.5;
	if(_Verbose)
		printf("-0.5*innerProduct: %e\n", data );
	data = exp( data );
	if(_Verbose)
		printf("exp(...): %e\n", data );
    data = coeff * data;
	if(_Verbose)
		printf("Final Prob: %e\n\n", data);

	return data;
}

void Gaussian::show() {
	printf("Gaussian:\n");
	printf("Label: %s\n", label);
	printf("Dimension: %d\n", dimension);
	printf("Prior: %.2f\n", prior);
	printf("Sigma_det: %e\n", sigma_det);
	printf("u:\n");
	if(u != NULL)
		u->show();
	else
		printf(" Empty\n");
	printf("Sigma_inv:\n");
	if(u != NULL)
		sigma_inv->show();
	else
		printf(" Empty\n");
}

