/*****************************************************************************
 *
 * Author: Steve Dunn, MIT Wearable Computing Group
 * http://www.media.mit.edu/wearables/
 * Copyright (C) 2002 MIT Media Lab
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <stdlib.h>
#include <stdio.h>

#include "IntVector.hpp"

IntVector::IntVector() {
	numElements = 0;
	element = NULL;
}

IntVector::IntVector(int numVals) {
	int i;

	numElements = numVals;
	element = new long[numElements];
	if(element == NULL) {
		printf("Error allocation %d elements in vector\n", numElements);
		exit(1);
	}

	for(i=0; i<numElements; i++) 
		element[i] = 0;
}

IntVector::~IntVector() {
	if(numElements > 0)
		delete[] element;
}

//--------------------------------------------------

int IntVector::getNumElements() {
	return numElements;
}

long IntVector::getElement(int index) {
	if(index < 0 || index >= numElements)
		return 0;
	else
		return element[index];
}

void IntVector::setElement(int index, long val) {
	if(index < 0 || index >= numElements)
		return;

	element[index] = val;
}

void IntVector::appendElement(long val) {
	long *temp;
	int i;

	temp = new long[numElements+1];
	
	for(i=0; i<numElements; i++)
		temp[i] = element[i];

	if(numElements > 0)
		delete[] element;
	element = temp;
	temp[numElements] = val;
	numElements++;
}

//--------------------------------------------------

void IntVector::show() {
	int i;

	// printf("IntVector::show()\n");
    // printf("  numElements: %d\n", numElements);

	printf("Vect: ");
	for(i=0; i<numElements; i++) {
		printf("%ld ", element[i]);
	}
	printf("\n");
}

