/*****************************************************************************
 *
 * Author: Steve Dunn, MIT Wearable Computing Group
 * http://www.media.mit.edu/wearables/
 * Copyright (C) 2002 MIT Media Lab
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <stdlib.h>
#include <stdio.h>

#include "Matrix.hpp"
#include "RealVector.hpp"

int main(int argc, char *argv[]) {
	int i, j;
	int len = 3;
	RealVector *v1;
	RealVector *v2;
	RealVector *v3;
	Matrix *m1;
	Matrix *m2;
	Matrix *m3;

	printf("Linalg\n");

	v1 = new RealVector(len);
	for(i=0; i<len; i++)
		v1->setElement(i, i+1);
	v2 = new RealVector(len);
	for(i=0; i<len; i++)
		v2->setElement(i, i+4);
	v3 = new RealVector(len);
	m1 = new Matrix(len, len);
	for(i=0; i<len; i++)
		for(j=0; j<len; j++)
			m1->setElement(i, j, 10*(i+1)+(j+1));
	m2 = new Matrix(len, len);
	for(i=0; i<len; i++)
		for(j=0; j<len; j++)
			if(i == j)
				m2->setElement(i, j, 1);
			else
				m2->setElement(i, j, 0);
	m3 = new Matrix(len, len);

	printf("v1\n");
	v1->show();

	printf("v2\n");
	v2->show();

	printf("m1\n");
	m1->show();

	printf("m2\n");
	m2->show();

	printf("v1 . v2 = %.2f\n", RealVector::innerProduct(v1, v2));

	RealVector::outerProduct(v1, v2, m3);
	printf("v1 x v2\n");
	m3->show();

	Matrix::multiply(m1, m2, m3);
	printf("m1 x m2\n");
	m3->show();

	RealVector::multiply(v1, m1, v3);
	printf("v1 x m1\n");
	v3->show();

	delete v1;
	delete v2;
	delete m1;

	return 0;
}

