/*****************************************************************************
 *
 * Author: Steve Dunn, MIT Wearable Computing Group
 * http://www.media.mit.edu/wearables/
 * Copyright (C) 2002 MIT Media Lab
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>

#include "Context.hpp"
#include "ContextHandler.hpp"
#include "Signal.hpp"
#include "RealVector.hpp"
#include "Classification.hpp"
#include "Gaussian.hpp"
#include "MixtureModel.hpp"
#include "IntVector.hpp"
#include "Markov.hpp"
#include "MarkovList.hpp"

int main(int argc, char *argv[]) {
	char *sigUNL; // (ex. /signals/accel)
	char *gmmUNL;   // (ex. /classifiers/accel)
	char *mmUNL;   // (ex. /classifiers/accel2)

	Context *context = NULL;
	Signal *signal;
	RealVector *feature;
	Classification *gmmClassification;
	Classification *classification;
	IntVector *states;
	int id, i;
	unsigned long count;

	MixtureModel mixtureModel;
	MarkovList markovList;
	char *gmmDataFile;
	char *mmDataFile;

	printf("MGClassifier\n");
    if(argc < 7) {
        printf("Usage: MGClassifier [server name] [gaussian datafile] [markov datafile] [input signal] [gmm output unl] [mm output unl]\n");
        exit(0);
    }

    context = new Context(argv[1]);
	gmmDataFile = argv[2];
	mmDataFile = argv[3];
	sigUNL = argv[4];
	gmmUNL = argv[5];
	mmUNL = argv[6];
	
	mixtureModel.load(gmmDataFile);
	markovList.load(mmDataFile);

	gmmClassification = new Classification();
	classification = new Classification();
	signal = new Signal(sigUNL, context);
	states = new IntVector(markovList.getWindowSize());

	count = 0;
	printf("Reading Signals --------\n");
	while(1) {
		if( (feature = signal->readReals()) != NULL ) {
			// feature->show();

			// process feature vector here
			mixtureModel.classify(gmmClassification, feature);
			if(1 && context != NULL) {
				gmmClassification->publish(context, gmmUNL);
			}
			// gmmClassification->show();
			// printf("GMM Activity: %s\n", gmmClassification->mostLikely());

			id = gmmClassification->mostLikelyIndex();

			for(i=0; i<markovList.getWindowSize()-1; i++) 
				states->setElement(i, states->getElement(i+1));
			states->setElement(i, id);
				
			markovList.classify(classification, states);
			if(1 && context != NULL) {
				classification->publish(context, mmUNL);
			}
			// printf("Activity: %s\n", classification->mostLikely());

			// printf("%d %d\n", gmmClassification->mostLikelyIndex()+1, classification->mostLikelyIndex()+1);
			delete feature;

			// if((count % 100) == 0)
				// fprintf(stderr, "%ld entries\n", count);
			// count++;
		}
		else {
			printf("Sleeping\n");
			sleep(1);
		}

	}

	if(context != NULL) 
		delete context;
	delete signal;
	delete gmmClassification;
	delete classification;

	return 0;
}

