/*****************************************************************************
 *
 * Author: Steve Dunn, MIT Wearable Computing Group
 * http://www.media.mit.edu/wearables/
 * Copyright (C) 2002 MIT Media Lab
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "Markov.hpp"
#include "RealVector.hpp"
#include "Matrix.hpp"
#include "IntVector.hpp"

Markov::Markov() {
	identifier = 0;
	label = NULL;
	prior = 0.0;
	windowSize = 0;
	numElements = 0;
	initProb = NULL;
	transProb = NULL;
}

Markov::~Markov() {
}

double Markov::calc(IntVector *states) {
	int i;
	double val;

	val = 1.0;
	// val *= initProb->getElement(states->getElement(0));
	for(i=1; i<states->getNumElements(); i++) {
		val *= transProb->getElement(states->getElement(i-1), states->getElement(i));
		
// printf("%2d -- Val: %e\n", i, val);
// printf("%ld - %ld : %f\n", states->getElement(i-1), states->getElement(i), transProb->getElement(states->getElement(i-1), states->getElement(i)));
	}
// printf("\n");

	return val;
}

int Markov::getWindowSize() {
	return windowSize;
}

void Markov::show() {
	printf("Markov: %s\n", label);
	printf("NumElements: %d\n", numElements);
	printf("Window Size: %d\n", windowSize);
	printf("Prior: %f\n", prior);
	
	printf("Init Probability:\n");
	if(initProb != NULL)
		initProb->show();
	else
		printf(" Empty\n");

	printf("Trans Probability:\n");
	if(transProb != NULL)
		transProb->show();
	else
		printf(" Empty\n");
}

void Markov::normalizeProbabilities() {
	int row, col;
	double total;

	total = 0.0;
	for(col=0; col<numElements; col++)
		total += initProb->getElement(col);
	for(col=0; col<numElements; col++) {
		if(total > 0)
			initProb->setElement(col, initProb->getElement(col)/total);
		else
			initProb->setElement(col, MIN_PROB);
	}

	for(row=0; row<numElements; row++) {
		total = 0.0;
		for(col=0; col<numElements; col++)
			total += transProb->getElement(row, col);
		for(col=0; col<numElements; col++) {
			if(total > 0)
				transProb->setElement(row, col, transProb->getElement(row, col)/total);
			else
				transProb->setElement(row, col, MIN_PROB);
		}
	}
}

