/*****************************************************************************
 *
 * Author: Steve Dunn, MIT Wearable Computing Group
 * http://www.media.mit.edu/wearables/
 * Copyright (C) 2002 MIT Media Lab
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <stdlib.h>
#include <stdio.h>

#include "Matrix.hpp"

/*
Matrix::Matrix() {
	numElements = 0;
	element = NULL;

	numRows = 0;
	numCols = 0;
}
*/

Matrix::Matrix(int rows, int cols) {
    int i;

	numRows = rows;
	numCols = cols;
    numElements = numRows*numCols;
    element = new double[numElements];

    for(i=0; i<numElements; i++) 
        element[i] = 0;
}

Matrix::~Matrix() {
	if(numElements > 0)
		delete[] element;
}

//--------------------------------------------------

int Matrix::getNumElements() {
	return numElements;
}

int Matrix::getNumRows() {
	return numRows;
}

int Matrix::getNumCols() {
	return numCols;
}

double Matrix::getElement(int row, int col) {
	if(row < 0 || row >= numRows || col < 0 || col >= numCols)
		return 0;
	else
		return element[row*numRows+col];
}

void Matrix::setElement(int row, int col, double val) {
	if(row < 0 || row >= numRows)
		return;

	if(col < 0 || col >= numCols)
		return;

	element[row*numRows+col] = val;
}

//--------------------------------------------------

void Matrix::show() {
	int i, j;

	// printf("Matrix::show()\n");
    // printf("rows: %d cols: %d\n", numRows, numCols);

	printf("Matrix:\n");
	for(i=0; i<numRows; i++) {
		for(j=0; j<numCols; j++) {
    		printf("%e ", element[numRows*i+j]);
    		// printf("%.2f ", element[numRows*i+j]);
		}
		printf("\n");
	}
}

//--------------------------------------------------

void Matrix::identity() {
	int i, j;

	for(i=0; i<numRows; i++) {
		for(j=0; j<numCols; j++) {
			if(i == j)
				element[numRows*i+j] = 1;
			else
				element[numRows*i+j] = 0;
		}
	}
}

//--------------------------------------------------

void Matrix::multiply(Matrix *m1, Matrix *m2, Matrix *result) {
	int i, j, k;
	double sum;

	if(m1->numCols != m2->numRows) {
		printf("Error - mismatched matrix sizes for multiply\n");
		return;
	}

	if(m1->numRows != result->numRows || result->numCols != m2->numCols) {
		printf("Error - mismatched result matrix size for multiply\n");
		return;
	}

	for(i=0; i<m1->numRows; i++) {
		for(j=0; j<m2->numCols; j++) {
			sum = 0;
			for(k=0; k<m1->numCols; k++) {
				sum += m1->element[i*m1->numRows+k]*m2->element[k*m2->numRows+j];
			}
			result->setElement(i, j, sum);
		}
	}
}

