/*****************************************************************************
 *
 * Author: Steve Dunn, MIT Wearable Computing Group
 * http://www.media.mit.edu/wearables/
 * Copyright (C) 2002 MIT Media Lab
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <stdlib.h>
#include <stdio.h>

#include "Ant.hpp"
#include "Classification.hpp"
#include "Trigger.hpp"
#include "Action.hpp"
#include "ActionMsg.hpp"
#include "ActionImage.hpp"
#include "ActionVisualMsg.hpp"

typedef struct _TriggerList {
	Trigger *trigger;
	struct _TriggerList *next;
} TriggerList;

TriggerList* loadTriggers(char *filename, const char *serverName);
void saveTriggers(char *filename, TriggerList *list);

class MyHandler : public ContextHandler {
protected:
    Classification classification;
public:
    Label *label;
	Image *image;
	Container *visBox;
	Image *visImage;
	Label *visLabel;
	SwitchPane *pane;
    TriggerList *list;

    MyHandler() { list = NULL; };
    virtual ~MyHandler() {}
    virtual void handleUpdate(NodeType *node, char *unl);
};

void MyHandler::handleUpdate(NodeType *updateNode, char *unl) {
	TriggerList *item;

    if(updateNode == NULL || pane == NULL)
        return;

	item = list;
	while(item != NULL) {
		// printf("node: %s / unl: %s\n", unl, item->trigger->getUnl());
    	if(strcmp(unl, item->trigger->getUnl()) == 0) {
        	classification.load(updateNode);
			// printf("Found trigger\n");

			if(item->trigger->isActive(&classification)) {
				printf("Found Active item->trigger: %s - %s\n", item->trigger->getUnl(), item->trigger->getLabel());
				if(item->trigger->action != NULL) {
					// printf("Initiating Action %d\n", item->trigger->action->type);
					item->trigger->action->show();
					if(item->trigger->action->type == Action::ACTION_MSG) {
						if(label != NULL) {
							// printf("Msg Action - '%s'\n", ((ActionMsg*) item->trigger->action)->getMsg());
							label->setVal(((ActionMsg*) item->trigger->action)->getMsg());
							pane->setActivePane(label);
						}
					}
					else if(item->trigger->action->type == Action::ACTION_IMG) {
						if(image != NULL) {
							// printf("Image Action - image '%s'\n", ((ActionImage*) item->trigger->action)->getFilename());
							image->loadImage( ((ActionImage*) item->trigger->action)->getFilename());
							pane->setActivePane(image);
						}
					}
					else if(item->trigger->action->type == Action::ACTION_VISUAL_MSG) {
						if(visImage != NULL && visLabel != NULL && visBox != NULL) {
							// printf("VisualMsg  Action - image '%s' - '%s'\n", ((ActionVisualMsg*) item->trigger->action)->getFilename(), ((ActionVisualMsg*) item->trigger->action)->getMsg());
							visImage->loadImage( ((ActionVisualMsg*) item->trigger->action)->getFilename());
							visLabel->setVal(((ActionVisualMsg*) item->trigger->action)->getMsg());
							pane->setActivePane(visBox);
						}
					}
					else {
						printf("Unknown Action\n");
					}
				}
			}
		}
		
		item = item->next;
	}
} 

int main(int argc, char *argv[]) {
	MainApp mainApp;
	MainWin mainWin;
	Label msg;
	Image image;
	// Classification *classification;
	TriggerList *list;
	TriggerList *item;
	VBox vBox;
	SwitchPane pane;
    MyHandler handler;
	VBox visBox;
	Image visImage;
	Label visLabel;

    if(argc >= 2) {
        mainApp.setServerName(argv[1]);
    }
    else {
        printf("Usage: Notifier [server name]\n");
        exit(0);
    }

    if(mainApp.context != NULL)
        mainApp.setContextHandler(&handler);

	mainWin.setName("Notifier");
	mainWin.addChild(&vBox);
	vBox.addChild(&pane);

	image.loadImage("test.ppm");
	pane.addChild(&image);

	visImage.loadImage("test.ppm");
	visBox.addChild(&visImage);
	visLabel.setVal("VisualMsg");
	visBox.addChild(&visLabel);
	pane.addChild(&visBox);

	msg.setVal("Notifier App");
	pane.addChild(&msg);

	list = loadTriggers("triggers.in", mainApp.getServerName());
	handler.label = &msg;
	handler.list = list;
	handler.image = &image;
	handler.pane = &pane;
	handler.visBox = &visBox;
	handler.visImage = &visImage;
	handler.visLabel = &visLabel;

	item = list;
	while(item != NULL) {
		if(mainApp.context != NULL) 
			mainApp.context->subscribeToNode(item->trigger->getUnl());

		// item->trigger->show();
		item = item->next;
	}

/*
	classification = new Classification();
	classification->appendElement("zing", 0.92);
	classification->show();

	item = list;
	while(item != NULL) {
		if(item->trigger->isActive(classification)) {
			printf("Active item->trigger: %s\n", item->trigger->getUnl());
			if(item->trigger->action != NULL)
				msg.setVal(item->trigger->action->getMsg());
		}
		item = item->next;
	}
	delete classification;
*/

	saveTriggers("triggers.out", list);

	mainWin.showAll();
	mainApp.mainLoop();

	while(list != NULL) {
		item = list;
		list = list->next;
		delete item->trigger;
		delete item;
	}

	return 0;
}

void saveTriggers(char *filename, TriggerList *list) {
	FILE *fp;
	TriggerList *item;

	fp = fopen(filename, "w");
	if(fp == NULL) {
		printf("Error - cannot open file\n");
		exit(1);
	}

	item = list;
	while(item != NULL) {
		if(item->trigger != NULL)
			item->trigger->write(fp);
		item = item->next;
	}

	fclose(fp);
}

TriggerList* loadTriggers(char *filename, const char *serverName) {
	TriggerList *item;
	TriggerList *list;
	FILE *fp;
	Trigger *myTrigger;

	fp = fopen(filename, "r");
	if(fp == NULL) {
		printf("Error - cannot open file\n");
		exit(1);
	}

	item = NULL;
	while( (myTrigger = Trigger::read(fp, serverName)) != NULL) {

		if(item == NULL) {
			item = new TriggerList();
			item->trigger = myTrigger;
			item->next = NULL;
			list = item;
		}
		else {
			item->next = new TriggerList();
			item = item->next;
			item->trigger = myTrigger;
			item->next = NULL;
		}

		// myTrigger->show();
	}

	fclose(fp);

	return list;
}

