/*****************************************************************************
 *
 * Author: Steve Dunn, MIT Wearable Computing Group
 * http://www.media.mit.edu/wearables/
 * Copyright (C) 2002 MIT Media Lab
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#ifndef REAL_VECTOR_HPP
#define REAL_VECTOR_HPP

#include "Matrix.hpp"

class RealVector {
protected:
	int numElements;

	double *element;

public:
	RealVector();
	RealVector(int numVals);
	virtual ~RealVector();

	virtual int getNumElements();
	virtual double getElement(int index);
	virtual void setElement(int index, double val);
	virtual void appendElement(double val);

	static void add(RealVector *v1, RealVector *v2, RealVector *result);
	static void add(RealVector *v1, double num, RealVector *result);
	static void subtract(RealVector *v1, RealVector *v2, RealVector *result);
	static void subtract(RealVector *v1, double num, RealVector *result);
	static void multiply(RealVector *v1, double num, RealVector *result);
	static double innerProduct(RealVector *v1, RealVector *v2);
	static void outerProduct(RealVector *v1, RealVector *v2, Matrix *result);

	static void multiply(RealVector *v1, Matrix *m1, RealVector *result);

	virtual void show();
};

#endif

