/*****************************************************************************
 *
 * Author: Steve Dunn, MIT Wearable Computing Group
 * http://www.media.mit.edu/wearables/
 * Copyright (C) 2002 MIT Media Lab
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>

#include "RealVector.hpp"
#include "SignalGen.hpp"
#include "Context.hpp"

int main(int argc, char *argv[]) {
	SignalGen *signalGen;
	RealVector *feature;
	char *sigUNL = "/signals/signal1";
	int i;
	Context *context = NULL;

    if(argc >= 2) {
        context = new Context(argv[1]);
    }
    else {
        printf("Usage: SampleSignal [server name]\n");
        // exit(0);
    }

	printf("SampleSignal\n");

	signalGen = new SignalGen(sigUNL, context);
	if(signalGen == NULL) {
		printf("Error creating SignalGen\n");
		exit(1);
	}

	feature = new RealVector(2);
	if(feature == NULL) {
		printf("Error creating feature\n");
		exit(1);
	}

	for(i=0; i<6; i++) {
		signalGen->getConnections(2000);

		if(signalGen->hasListeners()) {
			// read hypothetical sensor
			feature->setElement(0, 3);
			feature->setElement(1, i);

			signalGen->writeReals(feature);
		}
	}

	delete feature;
	delete signalGen;

	return 0;
}

