/*****************************************************************************
 *
 * Author: Steve Dunn, MIT Wearable Computing Group
 * http://www.media.mit.edu/wearables/
 * Copyright (C) 2002 MIT Media Lab
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#ifndef SIGNAL_GEN_HPP
#define SIGNAL_GEN_HPP

#include "RealVector.hpp"
#include "IntVector.hpp"

class Context;

class SignalGen {
protected:
	char *unl;
	int baseSocket;
	int socket_id;

	Context *context;

	static const unsigned long START_VAL = 0xaabbccdd;
	int numListeners;
	int *listeners;

	long ip;
	short port;

	virtual void addListener(int newSocket);
	virtual void removeListener(int newSocket);
	virtual void announceAddress();
	virtual void killConnection(int sock);
public:
	SignalGen();
	SignalGen(char *newUNL, Context *newContext);
	virtual ~SignalGen();

	virtual int hasListeners();
	virtual void startListener(char *newUNL, Context *newContext);
	virtual void getConnections(int waitMillisec);

	virtual void writeReals(RealVector *feature);
	virtual void writeInts(IntVector *feature);
};

#endif

