/*****************************************************************************
 *
 * Author: Steve Dunn, MIT Wearable Computing Group
 * http://www.media.mit.edu/wearables/
 * Copyright (C) 2002 MIT Media Lab
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/time.h>
#include <math.h>

#include <fcntl.h>
#include <errno.h>
#include <sys/ioctl.h>

#include "Context.hpp"

#include "IntVector.hpp"
#include "SignalGen.hpp"

int main(int argc, char *argv[]) {
	SignalGen *signalGen;
	IntVector *feature;
	char *sigUNL = "/signals/time";
	Context *context = NULL;
	struct timeb startTime, endTime;
	long loopTime = 20905l;
	long interval;

    if(argc >= 2) {
        context = new Context(argv[1]);
    }
    else {
        printf("Usage: TimeSignal [server name]\n");
        exit(0);
    }

	printf("TimeSignal\n");

	signalGen = new SignalGen(sigUNL, context);
	if(signalGen == NULL) {
		printf("Error creating SignalGen\n");
		exit(1);
	}

	feature = new IntVector(1);
	if(feature == NULL) {
		printf("Error creating feature\n");
		exit(1);
	}

	ftime(&startTime);
	while(1) {
		signalGen->getConnections(0);

		if(0 || signalGen->hasListeners()) {
			ftime(&endTime);
			interval = (endTime.time*1000+endTime.millitm) - (startTime.time*1000+startTime.millitm);

			// printf("Time: %ld - %ld - %ld\n", loopTime , interval, loopTime - (1000*interval));

			if( (loopTime - (1000*interval)) > 0)
				usleep(loopTime - 1000*interval);
			else
				fwrite("Over timeslice\n", 1, strlen("Over timeslice\n"), stdout);
			ftime(&startTime);

			feature->setElement(0, startTime.time);

feature->show();
			signalGen->writeInts(feature);
		}
		else {
			sleep(1);
		}
	}

	delete feature;
	delete signalGen;

	return 0;
}

