/*****************************************************************************
 *
 * Author: Steve Dunn, MIT Wearable Computing Group
 * http://www.media.mit.edu/wearables/
 * Copyright (C) 2002 MIT Media Lab
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "Trigger.hpp"
#include "Action.hpp"
#include "ActionMsg.hpp"
#include "Classification.hpp"

Trigger::Trigger() {
	unl = NULL;
	label = NULL;
	threshold = 0.0;

	action = NULL;
}

Trigger::Trigger(char *newUnl, char *newLabel, double newThreshold) {
	unl = strdup(newUnl);
	label = strdup(newLabel);
	threshold = newThreshold;

	action = NULL;
}

Trigger::Trigger(char *newUnl, char *newLabel, double newThreshold, char *newMsg) {
	unl = strdup(newUnl);
	label = strdup(newLabel);
	threshold = newThreshold;

	action = new ActionMsg(newMsg);
}

Trigger::~Trigger() {
	if(unl != NULL)
		free(unl);

	if(label != NULL)
		free(label);

	if(action != NULL)
		delete action;
}

//-----------------------------

void Trigger::setUnl(char *newUnl) {
	if(unl != NULL)
		free(unl);
	
	unl = strdup(newUnl);
}

void Trigger::setLabel(char *newLabel) {
	if(label != NULL)
		free(label);
	
	label = strdup(newLabel);
}

void Trigger::setThreshold(double newThreshold) {
	threshold = newThreshold;
}

void Trigger::setAction(Action *newAction) {
	if(action != NULL)
		delete action;

	action = newAction;
}

//-----------------------------

char* Trigger::getUnl() {
	return unl;
}

char* Trigger::getLabel() {
	return label;
}

double Trigger::getThreshold() {
	return threshold;
}

//-----------------------------

void Trigger::show() {
	printf("Trigger\n");
	if(unl != NULL)
		printf(" UNL: '%s'\n", unl);
	if(label != NULL) {
		printf(" Label: '%s'\n", label);
		printf(" Threshold: %.2f\n", threshold);
	}

	if(action == NULL)
		printf(" No Action\n");
	else
		action->show();
}

int Trigger::isActive(Classification *classification) {

	if(classification == NULL)
		return 0;

	return classification->meetsThreshold(label, threshold);
}

//-----------------------------

void Trigger::write(FILE *fp) {
	fprintf(fp, "%s\t\%s\t%f\t", unl, label, threshold);

	if(action == NULL)
		fprintf(fp, "%d", Action::ACTION_NONE);
	else
		action->write(fp);

	fprintf(fp, "\n");
}

//-----------------------------

int Trigger::split(char *data, char **field, int maxFields, char **remainder) {
	char *ptr;
	int i;

	for(i=0; i<maxFields; i++)
		field[i] = NULL;

	for(i=0; i<maxFields && data != NULL; i++) {
		ptr = strchr(data, '\t');

		field[i] = data;
		if(ptr != NULL) {
			*ptr = '\0';
			data = ptr+1;
		}
		else {
			data = NULL;
		}
	}

	if(remainder != NULL)
		*remainder = data;

	return i;
}

//-----------------------------

Trigger* Trigger::read(FILE *fp, const char *serverName) {
	char buffer[1024];
	char unl[512+1024];
	Trigger *trigger;
	char *ptr, *data;
	char *field[4];
	int i;
	int actionType;

	data = buffer;
	if(fgets(data, 1023, fp) == NULL)
		return NULL;

	ptr = strchr(data, '\n');
	if(ptr != NULL) 
		*ptr = '\0';
	// printf("Line: %s\n", data);

	i = split(data, field, 4, &ptr);

	if(i == 4) {
		if(Context::isRelativePath(field[0])) {
			sprintf(unl,"%s:%s", serverName, field[0]);
			trigger = new Trigger(unl, field[1], atof(field[2]));
		}
		else {
			trigger = new Trigger(field[0], field[1], atof(field[2]));
		}
		actionType = atol(field[3]);
		if(actionType != Action::ACTION_NONE) {
			trigger->setAction(Action::read(actionType, ptr));
		}
	}
	else {
		trigger = new Trigger();
	}

	return trigger;
}

